'use strict';
var self = normalizeStep;
module.exports = self;
const { supportedOsFamily, supportedOsAndFamilyMap, isSupportedOperatingSystem, OS_FAMILY_MACOS } = require('../nodeMetadataMap');
const { DEFAULT_STEP_TIMEOUT_SECONDS, TROUBLESHOOT_MSG } = require('../constants.js');
var _ = require('underscore');
const util = require('util');
const async = require('async');
function normalizeStep(externalBag, callback) {
    var bag = {
        step: _.clone(externalBag.step),
        isExternalStep: externalBag.isExternalStep,
        pipeline: _.clone(externalBag.pipeline),
        nodePools: externalBag.nodePools,
        defaultNodePoolId: externalBag.defaultNodePoolId,
        injectedEnvs: externalBag.injectedEnvs,
        injectedPipeEnvs: externalBag.injectedPipeEnvs,
        masterResource: externalBag.masterResource,
        isK8sNodePool: false,
        globalEnvironmentVariables: externalBag.globalEnvironmentVariables,
        projectEnvironmentVariables: externalBag.projectEnvironmentVariables,
        internalIntegrationsById: externalBag.internalIntegrationsById,
        consoleErrors: []
    };
    bag.who = util.format('runTrigger|common|%s|jFrogPipelinesSessionId:%s', self.name, externalBag.jFrogPipelinesSessionId);
    logger.debug('Inside', bag.who);
    async.series([
        _checkInputParams.bind(null, bag),
        _normalizeConfigNodePoolSection.bind(null, bag),
        _normalizeConfigTimeoutSection.bind(null, bag),
        _normalizeConfigAffinityGroupSection.bind(null, bag),
        _normalizeConfigRuntimeSection.bind(null, bag),
        _normalizeConfigEnvironmentVariablesSection.bind(null, bag),
        _normalizeConfigCLIVersion.bind(null, bag)
    ], function () {
        bag.step = _.extend(bag.step, {
            configPropertyBag: bag.stepConfig
        });
        var result = {
            normalizedStep: bag.step,
            nodePool: bag.nodePool
        };
        return callback(bag.consoleErrors, result);
    });
}
function _checkInputParams(bag, next) {
    var who = bag.who + '|' + _checkInputParams.name;
    logger.trace(who, 'Inside');
    bag.pipelineConfig = bag.pipeline && bag.pipeline.ymlConfigPropertyBag || {};
    bag.stepConfig = bag.step.configPropertyBag || {};
    bag.stepExecution = bag.step.execPropertyBag || {};
    return next();
}
function _normalizeConfigNodePoolSection(bag, next) {
    if (bag.isExternalStep)
        return next();
    var who = bag.who + '|' + _normalizeConfigNodePoolSection.name;
    logger.trace(who, 'Inside');
    var ymlNodePool = bag.stepConfig.nodePool;
    if (!ymlNodePool && bag.pipelineConfig.nodePool) {
        ymlNodePool = bag.pipelineConfig.nodePool;
        bag.stepConfig.nodePool = bag.pipelineConfig.nodePool;
    }
    if (ymlNodePool) {
        var nodePool = _.findWhere(bag.nodePools, { name: ymlNodePool });
        if (!nodePool)
            bag.consoleErrors.push(util.format('No nodePool found with the name %s. ' +
                'Add the nodePool with this name and trigger the step again.', ymlNodePool));
        else
            bag.nodePool = nodePool;
    }
    else {
        var defaultNodePool;
        if (bag.defaultNodePoolId)
            defaultNodePool = _.findWhere(bag.nodePools, { id: bag.defaultNodePoolId });
        if (!defaultNodePool) {
            bag.consoleErrors.push('No default nodePool is found. ' +
                'Set one of the nodePools as default and trigger the step again.');
        }
        else {
            bag.stepConfig.nodePool = defaultNodePool.name;
            bag.nodePool = defaultNodePool;
        }
    }
    if (bag.nodePool) {
        var stepType = _.findWhere(global.systemCodes, {
            code: bag.step.typeCode,
            group: 'resource'
        }).name;
        var nodePoolOS = bag.nodePool.operatingSystem;
        var nodePoolOsFamily = supportedOsAndFamilyMap[nodePoolOS];
        if (nodePoolOsFamily === OS_FAMILY_MACOS
            && bag.step.typeCode !== global.systemCodesByName.Bash.code) {
            bag.consoleErrors.push('Only steps of type "Bash" are supported on MacOS build nodes. ' +
                'https://www.jfrog.com/confluence/display/JFROG/Bash');
        }
        if (!isSupportedOperatingSystem(bag.nodePool.architecture, nodePoolOS)) {
            bag.consoleErrors.push(`${nodePoolOS} has reached end of support. ` +
                `Please change the OS to another supported version. ${TROUBLESHOOT_MSG}`);
        }
        if (nodePoolOsFamily === supportedOsFamily.WINDOWS &&
            stepType === 'Bash') {
            bag.consoleErrors.push('The selected node pool contains Windows nodes. ' +
                'Bash steps cannot run on a Windows node.');
        }
        else if (nodePoolOsFamily === supportedOsFamily.LINUX &&
            stepType === 'PowerShell') {
            bag.consoleErrors.push('The selected node pool does not contain ' +
                'Windows nodes. PowerShell steps can only run on a Windows node.');
        }
        else if (nodePoolOsFamily === supportedOsFamily.MACOS &&
            stepType === 'PowerShell') {
            bag.consoleErrors.push('The selected node pool contains MacOS nodes. ' +
                'PowerShell steps can only run on a Windows node.');
        }
        if (bag.masterResource) {
            var supportedOperatingSystems = _.pluck(bag.masterResource.platformsPropertyBag, 'os');
            if (!supportedOperatingSystems.includes(nodePoolOsFamily)) {
                var msg = util.format('Non-%s step cannot be executed in %s-type nodepool %s.', nodePoolOsFamily, nodePoolOsFamily, bag.nodePool.name);
                bag.consoleErrors.push(msg);
            }
        }
    }
    var nodePoolProjectIntegration = bag.nodePool &&
        bag.internalIntegrationsById &&
        bag.internalIntegrationsById[bag.nodePool.projectIntegrationId] || {};
    bag.isK8sNodePool = (nodePoolProjectIntegration.masterIntegrationName === 'kubernetesConfig');
    return next();
}
function _normalizeConfigTimeoutSection(bag, next) {
    var who = bag.who + '|' + _normalizeConfigTimeoutSection.name;
    logger.trace(who, 'Inside');
    var nodeTimeout;
    var timeoutLimitInSeconds = global.systemConfig.stepTimeoutMS / 1000;
    var defaultTimeout = DEFAULT_STEP_TIMEOUT_SECONDS;
    if (defaultTimeout > timeoutLimitInSeconds)
        defaultTimeout = timeoutLimitInSeconds;
    if (bag.nodePool && bag.nodePool.timeoutMS)
        nodeTimeout = bag.nodePool.timeoutMS / 1000;
    if (!bag.stepConfig.timeoutSeconds) {
        bag.stepConfig.timeoutSeconds = nodeTimeout || defaultTimeout;
    }
    else if (timeoutLimitInSeconds &&
        bag.stepConfig.timeoutSeconds > timeoutLimitInSeconds) {
        bag.consoleErrors.push(util.format('Step timeoutSeconds cannot exceed limit of %s seconds.', timeoutLimitInSeconds));
    }
    return next();
}
function _normalizeConfigAffinityGroupSection(bag, next) {
    var who = bag.who + '|' + _normalizeConfigAffinityGroupSection.name;
    logger.trace(who, 'Inside');
    var stepAffinityGroup = bag.stepConfig.affinityGroup ||
        bag.pipelineConfig.affinityGroup || bag.step.name;
    bag.step.affinityGroup = stepAffinityGroup;
    return next();
}
function _normalizeConfigRuntimeSection(bag, next) {
    if (!_.isEmpty(bag.consoleErrors))
        return next();
    if (bag.isExternalStep)
        return next();
    var who = bag.who + '|' + _normalizeConfigRuntimeSection.name;
    logger.trace(who, 'Inside');
    var os = bag.nodePool.operatingSystem;
    var pipelineRuntime = bag.pipelineConfig.runtime || {};
    var stepRuntime = bag.stepConfig.runtime || {};
    var runtime = {
        type: stepRuntime.type || pipelineRuntime.type,
        image: stepRuntime.image || pipelineRuntime.image || {}
    };
    if (!runtime.type) {
        if (os.startsWith('MacOS'))
            runtime.type = 'host';
        else
            runtime.type = 'image';
    }
    if (runtime.type !== 'host') {
        var imageName = '';
        var imageTag = '';
        var autoPull = true;
        var containerOptions = '';
        var image = runtime.image || {};
        var arch = bag.nodePool.architecture;
        if (os.startsWith('MacOS')) {
            bag.consoleErrors.push('The selected node pool contains MacOS nodes. ' +
                'Steps on a MacOS node must have runtime type host.');
            return next();
        }
        if (_.isEmpty(image.custom) && !(image.auto && image.auto.language)) {
            var stepType = _.findWhere(global.systemCodes, {
                code: bag.step.typeCode,
                group: 'resource'
            }).name;
            if (stepType === 'GoBuild' ||
                stepType === 'GoPublishBinary' || stepType === 'GoPublishModule') {
                image.auto = image.auto || {};
                image.auto.language = 'go';
            }
            else if (stepType === 'GradleBuild') {
                image.auto = image.auto || {};
                image.auto.language = 'java';
            }
            else if (stepType === 'MvnBuild') {
                image.auto = image.auto || {};
                image.auto.language = 'java';
            }
            else if (stepType === 'NpmBuild' || stepType === 'NpmPublish') {
                image.auto = image.auto || {};
                image.auto.language = 'node';
            }
        }
        if (!_.isEmpty(image.custom)) {
            var custom = image.custom;
            imageName = custom.name || '';
            imageTag = custom.tag || '';
            autoPull = _.isBoolean(custom.autoPull) ? custom.autoPull : true;
            containerOptions = custom.options || '';
            runtime.image = {
                imageName: imageName,
                imageTag: imageTag,
                containerOptions: containerOptions,
                autoPull: autoPull
            };
            if (custom.registry)
                runtime.image.registry = custom.registry;
            if (custom.sourceRepository)
                runtime.image.sourceRepository = custom.sourceRepository;
            if (custom.region)
                runtime.image.region = custom.region;
        }
        else if (image.auto && image.auto.language) {
            var language = image.auto.language.toString().toLowerCase();
            var langImage = _.find(global.config.languageImages, function (configImage) {
                return configImage.architecture === arch && configImage.os === os &&
                    configImage.language &&
                    configImage.language.toString().toLowerCase() === language;
            });
            imageName = langImage && langImage.image;
            if (_.isEmpty(imageName)) {
                bag.consoleErrors.push('No default image has been configured for ' +
                    'architecture: ' + arch + ', operating system: ' + os +
                    ', and language: ' + image.auto.language + '. ' +
                    'Please add a custom image to the runtime configuration for this ' +
                    'step in your YML file or see the documentation to configure a ' +
                    'default docker image for this language at the system level.');
                return next();
            }
            if (langImage.registryUrl)
                imageName = langImage.registryUrl + '/' + imageName;
            imageTag = '';
            if (!_.isEmpty(image.auto.versions))
                imageTag = image.auto.versions[0];
            else if (langImage.defaultVersion)
                imageTag = langImage.defaultVersion;
            runtime.image = {
                imageName: imageName,
                imageTag: imageTag
            };
        }
        else {
            var defaultLangImage = _.findWhere(global.config.languageImages, {
                architecture: arch,
                os: os,
                isDefault: true
            });
            imageName = defaultLangImage && defaultLangImage.image;
            if (_.isEmpty(imageName)) {
                bag.consoleErrors.push('No default image has been configured for ' +
                    'architecture: ' + arch + ' and operating system: ' + os + '. ' +
                    'Please add a custom image to the runtime configuration for this ' +
                    'step in your YML file or see the documentation to configure a ' +
                    'default docker image at the system level.');
                return next();
            }
            if (defaultLangImage.registryUrl)
                imageName = defaultLangImage.registryUrl + '/' + imageName;
            var defaultLanguageImageName = imageName;
            var defaultLanguageImageVersion = defaultLangImage &&
                defaultLangImage.defaultVersion || '';
            runtime.image = {
                imageName: defaultLanguageImageName,
                imageTag: defaultLanguageImageVersion
            };
        }
        bag.stepConfig = _.extend(bag.stepConfig, { runtime: runtime });
    }
    else {
        if (bag.isK8sNodePool) {
            bag.consoleErrors.push(util.format('The selected node pool: %s is a ' +
                'kubernetes node pool. Steps on a kubernetes node pool cannot run on ' +
                'host runtime. Please select the image runtime to run steps on a ' +
                'kubernetes node pool.', bag.nodePool.name));
            return next();
        }
        bag.stepConfig.runtime = {
            type: 'host'
        };
    }
    return next();
}
function _normalizeConfigEnvironmentVariablesSection(bag, next) {
    var who = bag.who + '|' + _normalizeConfigEnvironmentVariablesSection.name;
    logger.trace(who, 'Inside');
    var envs = [];
    var environmentVariables = bag.stepConfig.environmentVariables || {};
    var pipelineEnviromentVariables = bag.pipelineConfig.environmentVariables ||
        {};
    let envsByKey = {};
    var globalEnvironmentVariables = bag.globalEnvironmentVariables || [];
    _.each(globalEnvironmentVariables, function ({ key, value, isReadOnly }) {
        envs.push({
            key,
            value,
            isReadOnly,
            level: 'global'
        });
    });
    envsByKey = _.indexBy(envs, 'key');
    var projectEnvironmentVariables = bag.projectEnvironmentVariables || [];
    _.each(projectEnvironmentVariables, function ({ key, value, isReadOnly }) {
        let env = envsByKey[key];
        if (env) {
            env.key = key;
            env.value = value;
            env.isReadOnly = isReadOnly;
            env.level = 'project';
        }
        else {
            envs.push({
                key,
                value,
                isReadOnly,
                level: 'project'
            });
        }
    });
    envsByKey = _.indexBy(envs, 'key');
    if (!_.isEmpty(pipelineEnviromentVariables))
        if (!_.isEmpty(pipelineEnviromentVariables.readOnly))
            environmentVariables.readOnly = pipelineEnviromentVariables.readOnly;
    if (!_.isEmpty(environmentVariables) || !_.isEmpty(bag.injectedEnvs) ||
        !_.isEmpty(bag.injectedPipeEnvs)) {
        if (!_.isEmpty(environmentVariables.readOnly)) {
            _.each(environmentVariables.readOnly, function (value, key) {
                let defaultValue = value;
                let isSecure = false;
                if (typeof value === 'object') {
                    defaultValue = value.default;
                    isSecure = value.secure;
                }
                let env = envsByKey[key];
                if (env) {
                    env.value = defaultValue;
                    env.secure = isSecure;
                    env.isReadOnly = true;
                    env.level = 'pipeline';
                }
                else {
                    envs.push({
                        key: key,
                        value: defaultValue,
                        secure: isSecure,
                        isReadOnly: true,
                        level: 'pipeline'
                    });
                }
            });
        }
        envsByKey = _.indexBy(envs, 'key');
        if (!_.isEmpty(environmentVariables.normal)) {
            _.each(environmentVariables.normal, function (value, key) {
                let defaultValue = value;
                let allowCustom = true;
                let isSecure = false;
                if (typeof value === 'object') {
                    defaultValue = value.default;
                    allowCustom = value.allowCustom;
                    isSecure = value.secure;
                }
                let env = envsByKey[key];
                if (env) {
                    env.value = defaultValue;
                    env.isReadOnly = !allowCustom;
                    env.secure = isSecure;
                    env.level = 'step';
                }
                else {
                    envs.push({
                        key: key,
                        value: defaultValue,
                        isReadOnly: !allowCustom,
                        secure: isSecure,
                        level: 'step'
                    });
                }
            });
        }
        if (!_.isEmpty(bag.injectedPipeEnvs)) {
            _.each(bag.injectedPipeEnvs, function (injectedEnv) {
                var env = _.findWhere(envs, { key: injectedEnv.key });
                if (!env) {
                    envs.push({
                        key: injectedEnv.key,
                        value: injectedEnv.value,
                        level: 'pipeline'
                    });
                }
                else {
                    env.key = injectedEnv.key;
                    env.value = injectedEnv.value;
                    env.type = 'pipeline';
                }
            });
        }
        if (!_.isEmpty(bag.injectedEnvs)) {
            _.each(bag.injectedEnvs, function (injectedEnv) {
                var env = _.findWhere(envs, { key: injectedEnv.key });
                if (!env) {
                    envs.push({
                        key: injectedEnv.key,
                        value: injectedEnv.value,
                        level: 'step'
                    });
                }
                else {
                    env.key = injectedEnv.key;
                    env.value = injectedEnv.value;
                    env.type = 'step';
                }
            });
        }
        if (!_.isEmpty(environmentVariables.internal)) {
            _.each(environmentVariables.internal, function (value, key) {
                envs.push({
                    key: key,
                    value: value,
                    isInternal: true,
                    level: 'step'
                });
            });
        }
    }
    bag.stepConfig.environmentVariables = envs;
    return next();
}
function _normalizeConfigCLIVersion(bag, next) {
    var who = bag.who + '|' + _normalizeConfigCLIVersion.name;
    logger.trace(who, 'Inside');
    if (bag.pipelineConfig.jfrogCliVersion)
        bag.stepConfig.jfrogCliVersion = bag.pipelineConfig.jfrogCliVersion;
    return next();
}
//# sourceMappingURL=normalizeStep.js.map