"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.CreateHeadJson = void 0;
const node_util_1 = require("node:util");
const utility_1 = require("@jfrog-pipelines/utility");
const errors_1 = require("@jfrog-pipelines/errors");
const SyncRepository_1 = require("../../acl/syncService/SyncRepository");
const RunRepository_1 = require("../../RunRepository");
const constants_1 = require("../../constants");
const PIPELINE_OBJECTS_OBJECT_NAME = 'head';
const PIPELINE_OBJECTS_OBJECT_TYPE = 'run';
const PIPELINE_OBJECTS_STAGE = 'runTrigger';
class CreateHeadJson {
    constructor(payload) {
        this.run = payload.run;
        this.project = payload.project;
        this.pipelineSource = payload.pipelineSource;
        this.pipeline = payload.pipeline;
        this.projectIntegrationsById = payload.projectIntegrationsById;
        this.operationResources = payload.operationResources;
        this.masterIntegrationsById = payload.masterIntegrationsById;
        this.jFrogPipelinesSessionId = payload.jFrogPipelinesSessionId;
        this.parentRun = {};
        this.runDao = new RunRepository_1.RunDao();
        this.syncRespository = new SyncRepository_1.SyncRepository();
    }
    async createPipeInfo() {
        try {
            this.validate();
            const { template } = await this.getTemplateById();
            const parentRun = await this.getParentRunById();
            this.parentRun = {
                runNumber: parentRun.runNumber
            };
            const { pipelineName, pipelineSourceBranch } = await this.getParentPipelineById(parentRun.parentPipelineId);
            this.parentRun.pipelineName = pipelineName;
            if (pipelineSourceBranch) {
                this.parentRun.pipelineSourceBranch = pipelineSourceBranch;
            }
            const pipelineObjectBody = {
                objectId: this.run.id,
                objectName: PIPELINE_OBJECTS_OBJECT_NAME,
                objectType: PIPELINE_OBJECTS_OBJECT_TYPE,
                stage: PIPELINE_OBJECTS_STAGE,
                data: this.finalizeHeadJson((template ?? {}))
            };
            const pipelineObject = await this.syncRespository.postPipelineObjects(pipelineObjectBody);
            return pipelineObject;
        }
        catch (error) {
            throw new errors_1.ApplicationError(`createPipeInfo for request ${this.jFrogPipelinesSessionId} failed with ${(0, node_util_1.inspect)(error)}`);
        }
    }
    validate() {
        if ((0, utility_1.isEmpty)(this.run)) {
            throw new errors_1.BadRequestError('Run is empty');
        }
    }
    async getTemplateById() {
        try {
            if ((0, utility_1.isEmpty)(this.pipelineSource) || !this.pipelineSource.templateId) {
                return {};
            }
            const template = await this.syncRespository.getTemplateById(this.pipelineSource.templateId);
            return {
                template
            };
        }
        catch {
            return {};
        }
    }
    async getParentRunById() {
        try {
            const query = {
                where: {
                    id: this.run.parentRunId
                }
            };
            const { runNumber, pipelineId } = (await this.runDao.findOne(query));
            return {
                runNumber,
                parentPipelineId: pipelineId
            };
        }
        catch {
            return {};
        }
    }
    async getParentPipelineById(pipelineId) {
        if (!pipelineId) {
            return {};
        }
        const pipeline = await this.syncRespository.getPipelineById(pipelineId);
        const response = {
            pipelineName: pipeline.name
        };
        if (pipeline.pipelineSourceBranch && pipeline.pipelineSourceBranch.length > 0) {
            response.pipelineSourceBranch = pipeline.pipelineSourceBranch;
        }
        return response;
    }
    validateHeadJSON() {
        const checkList = [this.pipeline, this.project, this.pipelineSource, this.run];
        const isValid = checkList.every((item) => !(0, utility_1.isEmpty)(item));
        if (!isValid) {
            throw new errors_1.BadRequestError('invalid Head Json');
        }
    }
    finalizeHeadJson(template) {
        this.validateHeadJSON();
        const headJson = {
            pipelineName: this.pipeline.name,
            version: constants_1.PIPE_INFO_VERSION,
            projectName: this.project.name,
            pipelineSource: {
                repositoryFullName: this.pipelineSource.repositoryFullName,
                isMultiBranch: this.pipelineSource.isMultiBranch,
                branchName: this.pipeline.pipelineSourceBranch || this.pipelineSource.branch
            },
            run: {
                runNumber: this.run.runNumber,
                triggeredBy: {}
            },
            pipelinesYaml: {}
        };
        const projectIntegration = this.projectIntegrationsById[this.pipelineSource.projectIntegrationId];
        if (!(0, utility_1.isEmpty)(projectIntegration)) {
            headJson.pipelineSource.projectIntegration = {
                name: projectIntegration.name
            };
            const masterIntegration = this.masterIntegrationsById[projectIntegration.masterIntegrationId];
            if (!(0, utility_1.isEmpty)(masterIntegration)) {
                headJson.pipelineSource.projectIntegration.type = masterIntegration.displayName;
            }
        }
        if (!(0, utility_1.isEmpty)(template)) {
            headJson.pipelineSource.templateName = template.name;
            headJson.pipelineSource.templateNamespace = template.namespace;
        }
        if (!(0, utility_1.isEmpty)(this.run.parentRunId)) {
            headJson.run.triggeredBy.runNumber = this.parentRun.runNumber;
            headJson.run.triggeredBy.pipelineName = this.parentRun.pipelineName;
            if (!(0, utility_1.isEmpty)(this.parentRun.pipelineSourceBranch)) {
                headJson.run.triggeredBy.pipelineSourceBranch = this.parentRun.pipelineSourceBranch;
            }
        }
        if (this.run.staticPropertyBag.triggeredByUserName) {
            headJson.run.triggeredBy.username = this.run.staticPropertyBag.triggeredByUserName;
        }
        if (this.run.staticPropertyBag.triggeredByResourceName) {
            headJson.run.triggeredBy.resourceName = this.run.staticPropertyBag.triggeredByResourceName;
        }
        headJson.pipelinesYaml = {
            ...this.pipeline.ymlPropertyBag
        };
        headJson.pipelinesYaml.resources = [];
        for (const resource of this.operationResources) {
            if (!(0, utility_1.isEmpty)(resource.yml)) {
                headJson.pipelinesYaml.resources.push(resource.yml);
            }
        }
        return headJson;
    }
}
exports.CreateHeadJson = CreateHeadJson;
//# sourceMappingURL=createHeadJson.js.map