"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.CreateStepJson = void 0;
const node_util_1 = require("node:util");
const errors_1 = require("@jfrog-pipelines/errors");
const metadata_1 = require("@jfrog-pipelines/metadata");
const utility_1 = require("@jfrog-pipelines/utility");
const SyncRepository_1 = require("../../acl/syncService/SyncRepository");
const constants_js_1 = require("../../constants.js");
const SYSTEMCODES_BY_CODE = (0, metadata_1.getSystemCodesByCode)();
const INPUT_STEP_TYPE_FORMAT = '%s/%s(%s)';
const NODE_TYPES = {
    static: 'static',
    dynamic: 'dynamic'
};
const PIPELINE_OBJECTS_OBJECT_NAME = 'step';
const PIPELINE_OBJECTS_OBJECT_TYPE = 'step';
const PIPELINE_OBJECTS_STAGE = 'runTrigger';
class CreateStepJson {
    constructor(payload) {
        this.step = payload.step;
        this.runResourceVersions = payload.runResourceVersions;
        this.projectIntegrationsByName = payload.projectIntegrationsByName;
        this.masterIntegrationsById = payload.masterIntegrationsById;
        this.masterResourcesById = payload.masterResourcesById;
        this.pipelineStepsByName = payload.pipelineStepsByName;
        this.pipeline = payload.pipeline;
        this.pipelineSource = payload.pipelineSource;
        this.runNumber = payload.runNumber;
        this.nodePool = payload.nodePool;
        this.triggeredByIdentity = payload.triggeredByIdentity;
        this.jfrogPipelinesSessionId = payload.jfrogPipelinesSessionId;
        this.syncRespository = new SyncRepository_1.SyncRepository();
    }
    async createPipeInfo() {
        try {
            this.validate();
            const inputIntegrations = this.generateInputIntegrations();
            const inputSteps = this.generateInputSteps();
            const stepJson = this.finalizeStepJson(inputIntegrations, inputSteps);
            const pipelineObjectBody = {
                objectId: this.step.id,
                objectName: PIPELINE_OBJECTS_OBJECT_NAME,
                objectType: PIPELINE_OBJECTS_OBJECT_TYPE,
                stage: PIPELINE_OBJECTS_STAGE,
                data: stepJson
            };
            const pipelineObject = await this.syncRespository.postPipelineObjects(pipelineObjectBody);
            return pipelineObject;
        }
        catch (error) {
            throw new errors_1.ApplicationError(`createPipeInfo for request ${this.jfrogPipelinesSessionId} failed with ${(0, node_util_1.inspect)(error)}`);
        }
    }
    validate() {
        if ((0, utility_1.isEmpty)(this.step)) {
            throw new errors_1.BadRequestError('Step is empty');
        }
    }
    getIntegrationNames() {
        if ((0, utility_1.isEmpty)(this.step.configPropertyBag)) {
            return [];
        }
        if ((0, utility_1.isEmpty)(this.step.configPropertyBag.integrations)) {
            return [];
        }
        const { integrations = [] } = this.step.configPropertyBag;
        if ((0, utility_1.isEmpty)(integrations)) {
            return [];
        }
        return integrations.map((integration) => integration.name);
    }
    getDisplayName(projectIntegration) {
        const { masterIntegrationId } = projectIntegration;
        let displayName = '';
        if (!(0, utility_1.isEmpty)(this.masterIntegrationsById[masterIntegrationId])) {
            displayName = this.masterIntegrationsById[masterIntegrationId].displayName || '';
        }
        return displayName;
    }
    generateInputIntegrations() {
        const inputIntegrations = [];
        const integrationNames = this.getIntegrationNames();
        for (const integrationName of integrationNames) {
            const projectIntegration = this.projectIntegrationsByName[integrationName];
            if (!(0, utility_1.isEmpty)(projectIntegration)) {
                inputIntegrations.push({
                    name: projectIntegration.name,
                    type: this.getDisplayName(projectIntegration)
                });
            }
        }
        return inputIntegrations;
    }
    getInputStepNames() {
        if ((0, utility_1.isEmpty)(this.step.configPropertyBag)) {
            return [];
        }
        if ((0, utility_1.isEmpty)(this.step.configPropertyBag.inputSteps)) {
            return [];
        }
        const { inputSteps = [] } = this.step.configPropertyBag;
        if ((0, utility_1.isEmpty)(inputSteps)) {
            return [];
        }
        return inputSteps.map((inputStep) => inputStep.name);
    }
    getStepType(input) {
        const { masterResourceId, typeCode } = input;
        let stepType = SYSTEMCODES_BY_CODE[typeCode].name;
        if (masterResourceId) {
            const masterResource = this.masterResourcesById[masterResourceId];
            if (!(0, utility_1.isEmpty)(masterResource)) {
                const { namespace, name: masterResourceName, syntaxVersion } = masterResource;
                stepType = (0, node_util_1.format)(INPUT_STEP_TYPE_FORMAT, namespace, masterResourceName, syntaxVersion);
            }
        }
        return stepType;
    }
    generateInputSteps() {
        const inputSteps = [];
        const inputStepNames = this.getInputStepNames();
        for (const stepName of inputStepNames) {
            const pipelineStep = this.pipelineStepsByName[stepName];
            if (!(0, utility_1.isEmpty)(pipelineStep)) {
                const { name } = pipelineStep;
                const stepType = this.getStepType(pipelineStep);
                inputSteps.push({ name, type: stepType });
            }
        }
        return inputSteps;
    }
    getPipelineSourceBranch() {
        return this.pipelineSource.isMultiBranch ? this.pipeline.pipelineSourceBranch : this.pipelineSource.branch;
    }
    getStepJsonInputs(inputIntegrations, inputSteps) {
        const result = {};
        if (!(0, utility_1.isEmpty)(inputIntegrations)) {
            result.integrations = inputIntegrations;
        }
        if (!(0, utility_1.isEmpty)(inputSteps)) {
            result.steps = inputSteps;
        }
        return result;
    }
    getStepJsonNodePools() {
        if ((0, utility_1.isEmpty)(this.nodePool)) {
            return [];
        }
        const providerMetadataPropertyBag = (0, utility_1.isEmpty)(this.nodePool.providerMetadataPropertyBag)
            ? {}
            : this.nodePool.providerMetadataPropertyBag;
        const nodePool = {
            architecture: this.nodePool.architecture,
            operatingSystem: this.nodePool.operatingSystem,
            name: this.nodePool.name,
            type: this.nodePool.isOnDemand ? NODE_TYPES.dynamic : NODE_TYPES.static,
            isCacheEnabled: this.nodePool.isCacheEnabled,
            ...providerMetadataPropertyBag
        };
        return [nodePool];
    }
    getRunResourceVersionsByResourceVersionId() {
        const runResourceVersionsByResourceVersionId = {};
        for (const runResourceVersion of this.runResourceVersions) {
            runResourceVersionsByResourceVersionId[runResourceVersion.resourceVersionId] = runResourceVersion;
        }
        return runResourceVersionsByResourceVersionId;
    }
    getStepJsonTriggeredBy() {
        const triggeredBy = {};
        if (this.step.triggeredByIdentityId) {
            triggeredBy.username = this.triggeredByIdentity.name;
        }
        if (this.step.triggeredByResourceVersionId) {
            const runResourceVersionsByResourceVersionId = this.getRunResourceVersionsByResourceVersionId();
            const runResourceVersion = runResourceVersionsByResourceVersionId[this.step.triggeredByResourceVersionId];
            if (!(0, utility_1.isEmpty)(runResourceVersion)) {
                triggeredBy.resourceName = runResourceVersion.resourceName;
            }
        }
        return triggeredBy;
    }
    getStepJsonPipeline() {
        let pipeline = {};
        if (!(0, utility_1.isEmpty)(this.pipeline)) {
            pipeline = {
                name: this.pipeline.name,
                pipelineSourceBranch: this.getPipelineSourceBranch()
            };
        }
        return pipeline;
    }
    finalizeStepJson(inputIntegrations, inputSteps) {
        const stepJson = {
            name: this.step.name,
            version: constants_js_1.PIPE_INFO_VERSION,
            type: this.getStepType(this.step),
            inputs: this.getStepJsonInputs(inputIntegrations, inputSteps),
            nodePools: this.getStepJsonNodePools(),
            outputs: {},
            pipeline: this.getStepJsonPipeline(),
            runNumber: this.runNumber,
            triggeredBy: this.getStepJsonTriggeredBy()
        };
        if (!(0, utility_1.isEmpty)(this.step.affinityGroup)) {
            stepJson.affinityGroup = this.step.affinityGroup;
        }
        return stepJson;
    }
}
exports.CreateStepJson = CreateStepJson;
//# sourceMappingURL=createStepJson.js.map