"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.SyncRepository = void 0;
const node_util_1 = require("node:util");
const errors_1 = require("@jfrog-pipelines/errors");
const utility_1 = require("@jfrog-pipelines/utility");
const dependencyInjection_1 = require("../commons/dependencyInjection");
const StepRepository_1 = require("../stepService/StepRepository");
const SyncDataAdapterFactory_1 = require("./SyncDataAdapterFactory");
const PROJECT_ID = 'projectId';
const RESOURCE_ID = 'resourceId';
const IN = 'IN';
const OUT = 'OUT';
const PIPELINE_ID = 'pipelineId';
const PIPELINESTEP_ID = 'pipelineStepId';
const RUN_ID = 'runId';
class SyncRepository {
    constructor() {
        this.adapter = SyncDataAdapterFactory_1.SyncDataAdapterFactory.create(dependencyInjection_1.AclAdapterDI.api);
        this.stepRepository = new StepRepository_1.StepRepository();
    }
    async getProjectResourceResourceVersionPipelineStepConnection(resourceVersionIds) {
        let resourceVersionList = await this.getResourceVersions(resourceVersionIds);
        const [onlyPinnedResourceVersion, qualifiedResourceVersionsByResourceId, stepIdsToCheck] = await this.getPinnedQualifiedStepIdsTuple(resourceVersionList);
        resourceVersionList = onlyPinnedResourceVersion;
        const uniqueResourceIdsFromPinnedResourceVersion = [
            ...this.getUnique(onlyPinnedResourceVersion, RESOURCE_ID)
        ];
        const projectId = this.getProjectId(resourceVersionList);
        const [inResourceConnections, outResourceConnections] = await this.getResourceConnection(projectId, uniqueResourceIdsFromPinnedResourceVersion);
        const pipelineIds = this.getTriggerConnectedPipelineIds(inResourceConnections, outResourceConnections);
        let pipelines = [];
        let skipPipelineIdIfWaiting = [];
        if (pipelineIds.length > 0) {
            const result = await this.getSkipPipelineIdIfWaiting(pipelineIds);
            pipelines = result.pipelines;
            skipPipelineIdIfWaiting = result.skipPipelineIdIfWaiting;
        }
        return {
            stepIdsToCheck,
            qualifiedResourceVersionsByResourceId,
            resourceVersionList,
            inResourceConnections,
            outResourceConnections,
            pipelines,
            skipPipelineIdIfWaiting
        };
    }
    async getPipelineStepsByPipelineStepIds(pipelineStepIds) {
        return this.adapter.getPipelineSteps(pipelineStepIds);
    }
    async getQualifiedStepByPipelineStepIds(pipelineStepIds) {
        const pipelineSteps = await this.adapter.getPipelineSteps(pipelineStepIds);
        return pipelineSteps.map((pipelineStep) => pipelineStep.id);
    }
    async getMatchingRunIds(pipelineIds, inResourceConnections) {
        const connections = inResourceConnections.filter((connection) => pipelineIds.includes(connection.pipelineId));
        const pipelineStepIds = this.getUnique(connections, PIPELINESTEP_ID);
        const foundSteps = await this.stepRepository.getWaitingStepsByPipelineIds([...pipelineStepIds]);
        const pipelineStepIdsByPipelineId = {};
        const connectionsByPipelineId = (0, utility_1.groupBy)(connections, PIPELINE_ID);
        for (const pipelineId in connectionsByPipelineId) {
            pipelineStepIdsByPipelineId[pipelineId] = [
                ...this.getUnique(connectionsByPipelineId[pipelineId], PIPELINESTEP_ID)
            ];
        }
        const stepsByRunId = (0, utility_1.groupBy)(foundSteps, RUN_ID);
        const matchingRunIds = [];
        for (const runId in stepsByRunId) {
            const runSteps = stepsByRunId[runId];
            const { pipelineId } = runSteps[0];
            const pipelinePipelineStepIds = pipelineStepIdsByPipelineId[pipelineId];
            const stepPipelineStepIds = this.getUnique(runSteps, PIPELINESTEP_ID);
            if (pipelinePipelineStepIds.length === stepPipelineStepIds.size) {
                matchingRunIds.push(Number.parseInt(runId, 10));
            }
        }
        return matchingRunIds;
    }
    async postPipelineObjects(pipelineObject) {
        return this.adapter.postPipelineObjects(pipelineObject);
    }
    async getTemplateById(id) {
        return this.adapter.getTemplateById(id);
    }
    async getPipelineById(id) {
        return this.adapter.getPipelineById(id);
    }
    async getResourceVersions(resourceVersionIds) {
        const resourceVersionList = await this.adapter.getResourceVersions(resourceVersionIds);
        if (resourceVersionList.length === 0) {
            throw new errors_1.ApplicationError(`No resource version found for ids ${resourceVersionIds.join(',')}`);
        }
        return resourceVersionList;
    }
    getProjectId(resourceVersionList) {
        const uniqueProjectIds = this.getUnique(resourceVersionList, PROJECT_ID);
        if (uniqueProjectIds.size === 0) {
        }
        if (uniqueProjectIds.size > 1) {
            throw new errors_1.ApplicationError(`ResourceVersions cannot be more than one project, found ${(0, node_util_1.inspect)(uniqueProjectIds)}`);
        }
        const [projectId] = [...uniqueProjectIds];
        return projectId;
    }
    async getPinnedQualifiedStepIdsTuple(resourceVersionList) {
        const uniqueResourceIds = this.getUnique(resourceVersionList, RESOURCE_ID);
        if (uniqueResourceIds.size === 0) {
        }
        const resourceList = await this.adapter.getResources([...uniqueResourceIds]);
        const resourceMapById = resourceList.map((resource) => {
            return {
                [resource.id]: resource
            };
        });
        const onlyPinnedResourceVersion = resourceVersionList.filter((resourceVersion) => {
            const matchingResource = (resourceMapById[resourceVersion.resourceId] || {});
            return !matchingResource.pinnedVersionId;
        });
        const stepIdsToCheck = [];
        const qualifiedResourceVersions = [];
        for (const resourceVersion of onlyPinnedResourceVersion) {
            if (resourceVersion.createdByStepId) {
                stepIdsToCheck.push(resourceVersion.createdByStepId);
            }
            else {
                qualifiedResourceVersions.push(resourceVersion);
            }
        }
        const qualifiedResourceVersionsByResourceId = (0, utility_1.groupBy)(qualifiedResourceVersions, RESOURCE_ID);
        return [onlyPinnedResourceVersion, qualifiedResourceVersionsByResourceId, stepIdsToCheck];
    }
    async getResourceConnection(projectId, resourceIds) {
        const connections = await this.adapter.getPipelineStepConnection(projectId, resourceIds);
        const inResourceConnections = connections.filter((connection) => {
            return connection.operation === IN && connection.isTrigger;
        });
        const outResourceConnections = connections.filter((connection) => {
            return connection.operation === OUT;
        });
        return [inResourceConnections, outResourceConnections];
    }
    getTriggerConnectedPipelineIds(inResourceConnections, outResourceConnections) {
        const pipelineIds = [...this.getUnique(inResourceConnections, PIPELINE_ID)];
        const triggerConnectedPipelineIds = [];
        for (const pipelineId of pipelineIds) {
            const triggeringInConnections = inResourceConnections.filter((inResourceConnection) => {
                return inResourceConnection.pipelineId === pipelineId;
            });
            const connectionFromOtherPipeline = triggeringInConnections.some((triggeringInConnection) => {
                return outResourceConnections.some((outResourceConnection) => {
                    return (outResourceConnection.operationResourceId === triggeringInConnection.operationResourceId &&
                        outResourceConnection.pipelineId !== triggeringInConnection.pipelineId);
                });
            });
            if (connectionFromOtherPipeline) {
                triggerConnectedPipelineIds.push(pipelineId);
            }
        }
        return triggerConnectedPipelineIds;
    }
    async getSkipPipelineIdIfWaiting(pipelineIds) {
        const pipelines = await this.adapter.getPipelines(pipelineIds);
        const skipPipelineIdIfWaiting = [];
        for (const pipeline of pipelines) {
            if (pipeline.ymlConfigPropertyBag?.dependencyMode) {
                const dependencyMode = pipeline.ymlConfigPropertyBag.dependencyMode;
                if (this.isDependencyModeEnabled(dependencyMode)) {
                    skipPipelineIdIfWaiting.push(pipeline.id);
                }
            }
        }
        return {
            pipelines,
            skipPipelineIdIfWaiting
        };
    }
    isDependencyModeEnabled(dependencyMode) {
        return [
            dependencyMode.waitOnParentComplete,
            dependencyMode.waitOnChildComplete,
            dependencyMode.waitOnParentSuccess
        ].includes(true);
    }
    getUnique(list, attribute) {
        return new Set(list.map((attributes) => attributes[attribute]));
    }
}
exports.SyncRepository = SyncRepository;
//# sourceMappingURL=SyncRepository.js.map