"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const errors_1 = require("@jfrog-pipelines/errors");
jest.mock('../../stepService/StepRepository', () => {
    return {
        StepRepository: jest.fn().mockReturnValue({
            getWaitingStepsByPipelineIds: function (pipelineStepIds) {
                return Promise.resolve(pipelineStepIds);
            }
        })
    };
});
const SyncRepository_1 = require("../SyncRepository");
const ONE = 1;
const TWO = 2;
const SIX = 6;
function mockGetPipelineSteps(ids) {
    return Promise.resolve(ids.map((id) => ({ id })));
}
function mockGetResources(ids) {
    return Promise.resolve(ids.map((id) => {
        return {
            ymlConfigPropertyBag: {},
            systemPropertyBag: {},
            staticPropertyBag: {},
            yml: {
                name: 'intro_bag',
                type: 'PropertyBag',
                configuration: {
                    releaseVersion: ''
                }
            },
            id,
            name: 'intro_bag',
            pipelineSourceBranch: 'null',
            projectIntegrationId: 0,
            typeCode: 1034,
            masterResourceId: 0,
            latestResourceVersionId: 2,
            pinnedVersionId: 1,
            projectId: 1,
            pipelineSourceId: 1,
            isDeleted: false,
            deletedAt: new Date('2023-05-09T06:35:21.961Z'),
            isConsistent: true,
            isInternal: false,
            syntaxVersion: 'v1.0',
            nextTriggerTime: new Date('2023-05-09T06:35:21.961Z'),
            gitRepoFullName: '<gitRepoFullName>',
            gitIncludeBranchPattern: '<gitIncludeBranchPattern>',
            gitExcludeBranchPattern: '<gitExcludeBranchPattern>',
            gitBuildOnCommit: true,
            gitBuildOnPullRequestCreate: false,
            gitBuildOnPullRequestClose: false,
            gitBuildOnReleaseCreate: false,
            gitBuildOnTagCreate: false,
            storeVersionHistory: true,
            gitBuildOnBranchDelete: false,
            createdAt: '<createdAt>',
            updatedAt: '<updatedAt>'
        };
    }));
}
function mockGetResourceVersions(ids) {
    if (ids.length === 1 && ids[0] === ONE) {
        return Promise.resolve([]);
    }
    const result = ids.map((id) => {
        return {
            contentPropertyBag: {
                releaseVersion: 'intro_hello_world.2',
                timestamp: '1683614120',
                stepInfo: 'runNumber 2, step_name add_run_variable, pipeline_name intro_hello_world'
            },
            id: 2,
            projectId: 1,
            pipelineSourceId: 1,
            resourceId: id,
            createdByStepId: 6,
            createdByRunId: 2,
            createdAt: '<createdAt>',
            updatedAt: '<updatedAt>'
        };
    });
    return Promise.resolve(result);
}
function mockGetPipelineStepConnection(projectId, resourceIds) {
    const [operationResourceId] = resourceIds;
    return Promise.resolve([
        {
            id: 4,
            pipelineStepId: 3,
            operation: 'OUT',
            operationResourceId,
            operationResourceName: 'intro_bag',
            operationPipelineStepId: 1,
            operationPipelineStepName: '<operationPipelineStepName>',
            operationIntegrationId: 1,
            operationIntegrationName: '<operationIntegrationName>',
            projectId,
            pipelineSourceId: 1,
            pipelineId: 1,
            isPipelineConnection: false,
            isTrigger: false,
            createdBy: '1',
            updatedBy: '1'
        },
        {
            id: 6,
            pipelineStepId: 5,
            operation: 'IN',
            operationResourceId,
            operationResourceName: 'intro_bag',
            operationPipelineStepId: 1,
            operationPipelineStepName: '<operationPipelineStepName>',
            operationIntegrationId: 1,
            operationIntegrationName: '<operationIntegrationName>',
            projectId,
            pipelineSourceId: 1,
            pipelineId: 1,
            isPipelineConnection: false,
            isTrigger: true,
            createdBy: '1',
            updatedBy: '1'
        }
    ]);
}
jest.mock('../SyncDataAdapterFactory', () => {
    return {
        SyncDataAdapterFactory: {
            create: jest.fn().mockImplementation(() => {
                return {
                    getPipelineSteps: jest.fn().mockImplementation((ids) => mockGetPipelineSteps(ids)),
                    getResourceVersions: jest.fn().mockImplementation((ids) => mockGetResourceVersions(ids)),
                    getResources: jest.fn().mockImplementation((ids) => mockGetResources(ids)),
                    getPipelineStepConnection: jest
                        .fn()
                        .mockImplementation((projectId, resourceIds) => mockGetPipelineStepConnection(projectId, resourceIds))
                };
            })
        }
    };
});
const syncRepository = new SyncRepository_1.SyncRepository();
describe('getQualifiedStepByPipelineStepIds', () => {
    it('throws no error when passed with existing ids', async () => {
        await expect(syncRepository.getQualifiedStepByPipelineStepIds([ONE])).resolves.toEqual([ONE]);
    });
});
describe('getProjectResourceResourceVersionPipelineStepConnection', () => {
    it('throw error when resource version not found', async () => {
        const resourceVersionIds = [ONE];
        const expectedError = new errors_1.ApplicationError(`No resource version found for ids ${resourceVersionIds.join(',')}`);
        await expect(syncRepository.getProjectResourceResourceVersionPipelineStepConnection(resourceVersionIds)).rejects.toThrow(expectedError);
    });
    it('invoke with valid mocks', async () => {
        const resourceVersionIds = [TWO];
        const resourceVersion = await mockGetResourceVersions([TWO]);
        const resourceConnections = await mockGetPipelineStepConnection(ONE, [TWO]);
        const expectedResponse = {
            inResourceConnections: resourceConnections.filter((connection) => connection.operation === 'IN'),
            outResourceConnections: resourceConnections.filter((connection) => connection.operation === 'OUT'),
            resourceVersionList: resourceVersion,
            skipPipelineIdIfWaiting: [],
            stepIdsToCheck: [SIX],
            pipelines: [],
            qualifiedResourceVersionsByResourceId: {}
        };
        await expect(syncRepository.getProjectResourceResourceVersionPipelineStepConnection(resourceVersionIds)).resolves.toStrictEqual(expectedResponse);
    });
});
//# sourceMappingURL=SyncRepository.test.js.map