'use strict';
Object.defineProperty(exports, "__esModule", { value: true });
var self = microWorker;
module.exports = self;
const node_util_1 = require("node:util");
const configuration_1 = require("@jfrog-pipelines/configuration");
const logger_1 = require("@jfrog-pipelines/logger");
const SyncRepository_1 = require("./acl/syncService/SyncRepository");
const RunRepository_1 = require("./RunRepository");
var ApiAdapter = require('./Adapter.js');
var createRun = require('./_common/createRun.js');
var uuid = require('node-uuid');
const util = require('util');
const async = require('async');
var _ = require('underscore');
function microWorker(message, createRunCommand, callback) {
    var bag = {
        inPayload: message.payload,
        jFrogPipelinesSessionId: message.payload.jFrogPipelinesSessionId || uuid.v4(),
        inResourceConnections: [],
        outResourceConnections: [],
        skipPipelineIdIfWaiting: [],
        triggerablePipelineStepIds: [],
        triggerablePipelineSteps: [],
        triggersByPipelineStepId: {},
        pipelines: [],
        runs: []
    };
    var startTime = Date.now();
    bag.who = util.format('runTrigger|%s|jFrogPipelinesSessionId:%s|%s:%s', self.name, bag.jFrogPipelinesSessionId, createRunCommand.getType(), createRunCommand.getIds().join(','));
    logger.debug('Inside', bag.who);
    bag.suAdapter = new ApiAdapter(bag.jFrogPipelinesSessionId, configuration_1.ConfigFactory.config.apiToken);
    async.series([
        _getPipelineStepsByIds.bind(null, bag),
        _getProjectResourceResourceVersionPipelineStepConnection.bind(null, bag),
        _getWaitingRuns.bind(null, bag),
        _getVersionCreatedBySteps.bind(null, bag),
        _findStepsToTriggerForResource.bind(null, bag),
        _getResourceNames.bind(null, bag),
        _getPipelineSteps.bind(null, bag),
        _filterTriggerableStepsBasedOnDisabledPipelines.bind(null, bag),
        _createRuns.bind(null, bag)
    ], function (err) {
        var stopTime = Date.now();
        var timeDiff = new Date(stopTime - startTime);
        var minutes = timeDiff.getMinutes();
        var seconds = timeDiff.getSeconds();
        var ms = timeDiff.getMilliseconds();
        var duration = util.format('%s minutes, %s seconds, and ' +
            '%s milliseconds', minutes, seconds, ms);
        if (err)
            logger.warn(bag.who, util.format('Failed to process runTrigger message with error: %s. ' +
                'Time taken to process message is: %s', err, duration));
        else
            logger.debug(bag.who, util.format('Successfully processed runTrigger message.' +
                'Time taken to process message is: %s', duration));
        const eventData = {
            domain: 'run',
            eventType: 'runProcess',
            status: err ? 'failed' : 'success',
            durationMillis: timeDiff.valueOf(),
            jFrogPipelinesSessionId: bag.jFrogPipelinesSessionId
        };
        logger_1.LoggerFactory.logEvent(eventData);
        bag.runs.forEach((runData) => {
            logger_1.LoggerFactory.logEvent(runData);
            console.log(runData);
        });
        callback && callback();
    });
}
function _getPipelineStepsByIds(bag, next) {
    if (!bag.inPayload.pipelineStepIds &&
        !bag.inPayload.pipelineStepId) {
        return next();
    }
    const repository = new SyncRepository_1.SyncRepository();
    let queryStepIds = [];
    if (bag.inPayload.pipelineStepIds) {
        queryStepIds = bag.inPayload.pipelineStepIds;
    }
    if (bag.inPayload.pipelineStepId) {
        queryStepIds = bag.inPayload.pipelineStepId;
    }
    repository.getQualifiedStepByPipelineStepIds(queryStepIds).then((triggerablePipelineStepIds) => {
        bag.triggerablePipelineStepIds = triggerablePipelineStepIds;
        next();
    }).catch(error => {
        logger.warn(`Failed to getPipelineSteps for ids: ${(0, node_util_1.inspect)(queryStepIds.join(','))} with error`, (0, node_util_1.inspect)(error));
        next(error);
    });
}
function _getProjectResourceResourceVersionPipelineStepConnection(bag, next) {
    if (_.isEmpty(bag.inPayload.resourceVersionIds))
        return next();
    const repository = new SyncRepository_1.SyncRepository();
    repository.getProjectResourceResourceVersionPipelineStepConnection(bag.inPayload.resourceVersionIds)
        .then(({ stepIdsToCheck, qualifiedResourceVersionsByResourceId, resourceVersionList, inResourceConnections, outResourceConnections, pipelines, skipPipelineIdIfWaiting }) => {
        bag.stepIdsToCheck = stepIdsToCheck;
        bag.trigVersionsByResourceId = qualifiedResourceVersionsByResourceId;
        bag.resourceVersions = resourceVersionList;
        bag.inResourceConnections = inResourceConnections;
        bag.outResourceConnections = outResourceConnections;
        bag.pipelines = pipelines;
        bag.skipPipelineIdIfWaiting = skipPipelineIdIfWaiting;
        next();
    }).catch(error => {
        logger.warn(`Failed to getProjectResourceResourceVersionPipelineStepConnection for ids: ${(0, node_util_1.inspect)(bag.inPayload.resourceVersionIds)} with error`, (0, node_util_1.inspect)(error));
        next(error);
    });
}
function _getWaitingRuns(bag, next) {
    if (_.isEmpty(bag.skipPipelineIdIfWaiting))
        return next();
    var who = bag.who + '|' + _getWaitingRuns.name;
    logger.debug(who, 'Inside');
    var pipelineIdGroups = [];
    var i = 0;
    while (i < bag.skipPipelineIdIfWaiting.length) {
        pipelineIdGroups.push(bag.skipPipelineIdIfWaiting.slice(i, i + 10));
        i += 10;
    }
    const repository = new SyncRepository_1.SyncRepository();
    const runDao = new RunRepository_1.RunDao();
    async.eachLimit(pipelineIdGroups, 10, function (pipelineIdGroup, done) {
        let innerBag = {
            pipelineIds: pipelineIdGroup,
            inResourceConnections: bag.inResourceConnections,
            skippedPipelineIds: []
        };
        repository.getMatchingRunIds(innerBag.pipelineIds, innerBag.inResourceConnections).then(matchingRunIds => {
            innerBag.matchingRunIds = matchingRunIds;
            return runDao.getWaitingRunsForPipelineId(matchingRunIds);
        }).then((runs) => {
            const pipelineIds = _.pluck(runs, 'pipelineId');
            innerBag.skippedPipelineIds = _.uniq(innerBag.skippedPipelineIds.concat(pipelineIds));
            bag.inResourceConnections = _.filter(bag.inResourceConnections, function (connection) {
                return !_.contains(innerBag.skippedPipelineIds, connection.pipelineId);
            });
            return done();
        }).catch(err => {
            return done(err);
        });
    }, function (err) {
        return next(err);
    });
}
function _getVersionCreatedBySteps(bag, next) {
    if (_.isEmpty(bag.inPayload.resourceVersionIds))
        return next();
    if (_.isEmpty(bag.stepIdsToCheck))
        return next();
    var who = bag.who + '|' + _getVersionCreatedBySteps.name;
    logger.trace(who, 'Inside');
    var query = 'stepIds=' + bag.stepIdsToCheck.join(',');
    bag.suAdapter.getSteps(query, function (err, steps) {
        if (err) {
            logger.warn(who, util.format('Failed to getSteps for ids: %s with error: %s', bag.stepIdsToCheck.join(','), err));
            return next(true);
        }
        bag.versionCreatedBySteps = steps;
        return next();
    });
}
function _findStepsToTriggerForResource(bag, next) {
    if (_.isEmpty(bag.inPayload.resourceVersionIds))
        return next();
    var who = bag.who + '|' + _findStepsToTriggerForResource.name;
    logger.trace(who, 'Inside');
    if (_.isEmpty(bag.versionCreatedBySteps)) {
        bag.triggerablePipelineStepIds = _.pluck(bag.inResourceConnections, 'pipelineStepId');
        _.each(bag.inResourceConnections, function (connection) {
            var triggeringVersion = _.first(bag.trigVersionsByResourceId[connection.operationResourceId]);
            bag.triggersByPipelineStepId[connection.pipelineStepId] =
                {
                    resourceVersionId: triggeringVersion.id,
                    resourceId: triggeringVersion.resourceId
                };
        });
        return next();
    }
    _.each(bag.inResourceConnections, function (connection) {
        if (!_.isEmpty(bag.trigVersionsByResourceId[connection.operationResourceId])) {
            bag.triggerablePipelineStepIds.push(connection.pipelineStepId);
            var triggeringVersion = _.first(bag.trigVersionsByResourceId[connection.operationResourceId]);
            bag.triggersByPipelineStepId[connection.pipelineStepId] =
                {
                    resourceVersionId: triggeringVersion.id,
                    resourceId: triggeringVersion.resourceId
                };
        }
        else {
            _.each(bag.versionCreatedBySteps, function (vcbStep) {
                if (connection.pipelineId !== vcbStep.pipelineId) {
                    bag.triggerablePipelineStepIds.push(connection.pipelineStepId);
                    var trigger = _.findWhere(bag.resourceVersions, {
                        createdByStepId: vcbStep.id,
                        resourceId: connection.operationResourceId
                    });
                    if (!_.isEmpty(trigger))
                        bag.triggersByPipelineStepId[connection.pipelineStepId] =
                            {
                                resourceVersionId: trigger.id,
                                resourceId: trigger.resourceId
                            };
                }
            });
        }
    });
    bag.triggerablePipelineStepIds = _.uniq(bag.triggerablePipelineStepIds);
    return next();
}
function _getResourceNames(bag, next) {
    if (_.isEmpty(bag.triggersByPipelineStepId))
        return next();
    var who = bag.who + '|' + _getResourceNames.name;
    logger.debug(who, 'Inside');
    var resourceIds = _.compact(_.pluck(_.values(bag.triggersByPipelineStepId), 'resourceId'));
    if (_.isEmpty(resourceIds))
        return next();
    var resourceIdGroups = [];
    var i = 0;
    while (i < resourceIds.length) {
        resourceIdGroups.push(resourceIds.slice(i, i + global.config.defaultBatchSize));
        i += global.config.defaultBatchSize;
    }
    var resources = [];
    async.eachLimit(resourceIdGroups, 10, function (resourceIdGroup, done) {
        var query = util.format('resourceIds=%s&limit=%s', resourceIdGroup.join(','), global.config.defaultBatchSize);
        bag.suAdapter.getResources(query, function (err, response) {
            if (err) {
                logger.warn(who, util.format('Failed to getResources for ids: %s with error: %s', resourceIdGroup.join(','), err));
                return done(true);
            }
            resources = resources.concat(response);
            return done();
        });
    }, function (err) {
        if (err)
            return next(err);
        var stepsById = _.indexBy(bag.versionCreatedBySteps, 'id');
        var resourceVersionsById = _.indexBy(bag.resourceVersions, 'id');
        var resourcesById = _.indexBy(resources, 'id');
        _.each(bag.triggersByPipelineStepId, function (triggeringVersion) {
            var resource = resourcesById[triggeringVersion.resourceId];
            triggeringVersion.resourceName = resource && resource.name;
            var resourceVersion = resourceVersionsById[triggeringVersion.resourceVersionId];
            if (!resourceVersion)
                return;
            var triggeringStep = stepsById[resourceVersion.createdByStepId];
            if (!triggeringStep)
                return;
            triggeringVersion.parentRunId = triggeringStep.runId;
        });
        return next();
    });
}
function _getPipelineSteps(bag, next) {
    if (_.isEmpty(bag.triggerablePipelineStepIds))
        return next();
    var who = bag.who + '|' + _getPipelineSteps.name;
    logger.debug(who, 'Inside');
    var pipelineStepIdGroups = [];
    var i = 0;
    while (i < bag.triggerablePipelineStepIds.length) {
        pipelineStepIdGroups.push(bag.triggerablePipelineStepIds.slice(i, i + global.config.defaultBatchSize));
        i += global.config.defaultBatchSize;
    }
    bag.triggerablePipelineSteps = [];
    async.eachLimit(pipelineStepIdGroups, 10, function (pipelineStepIdGroup, done) {
        var query = util.format('pipelineStepIds=%s&limit=%s', pipelineStepIdGroup.join(','), global.config.defaultBatchSize);
        bag.suAdapter.getPipelineSteps(query, function (err, response) {
            if (err) {
                logger.warn(who, util.format('Failed to getPipelineSteps for ids: %s with error: %s', pipelineStepIdGroup.join(','), err));
                return done(true);
            }
            bag.triggerablePipelineSteps =
                bag.triggerablePipelineSteps.concat(response);
            return done();
        });
    }, function (err) {
        if (err)
            return next(err);
        return next();
    });
}
function _filterTriggerableStepsBasedOnDisabledPipelines(bag, next) {
    if (_.isEmpty(bag.triggerablePipelineSteps)) {
        return next();
    }
    const who = bag.who + '|' + _filterTriggerableStepsBasedOnDisabledPipelines.name;
    logger.debug(who, 'Inside');
    let pipelineIds = new Set();
    bag.triggerablePipelineSteps.forEach(function (eachTriggerableStep) {
        pipelineIds.add(eachTriggerableStep.pipelineId);
    });
    bag.pipelines.forEach(function (pipeline) {
        if (pipelineIds.has(pipeline.id)) {
            pipelineIds.delete(pipeline.id);
        }
    });
    const innerBag = {
        who: bag.who,
        suAdapter: bag.suAdapter,
        pipelineIds: [...pipelineIds],
        pipelines: bag.pipelines,
        disabledPipelineIds: [],
        disabledPipelineEntities: []
    };
    async.series([
        __getPipelinesDetails.bind(null, innerBag),
        __getDisabledPipelineEntities.bind(null, innerBag),
        __checkIfPipelineDisabled.bind(null, innerBag)
    ], function (err) {
        if (err) {
            return next(err);
        }
        const filteredTriggerablePipelineSteps = bag.triggerablePipelineSteps.filter(function (eachTriggerableStep) {
            return !(innerBag.disabledPipelineIds.includes(eachTriggerableStep.pipelineId));
        });
        bag.triggerablePipelineSteps = filteredTriggerablePipelineSteps;
        return next();
    });
}
function __getPipelinesDetails(innerBag, next) {
    if (_.isEmpty(innerBag.pipelineIds)) {
        return next();
    }
    const who = innerBag.who + '|' + __getPipelinesDetails.name;
    logger.debug(who, 'Inside');
    const { pipelineIds } = innerBag;
    let pipelineIdGroups = [];
    let i = 0;
    while (i < pipelineIds.length) {
        pipelineIdGroups.push(pipelineIds.slice(i, i + global.config.defaultBatchSize));
        i += global.config.defaultBatchSize;
    }
    async.eachLimit(pipelineIdGroups, 10, function (pipelineIdGroup, done) {
        const query = util.format('pipelineIds=%s&limit=%s', pipelineIdGroup.join(','), global.config.defaultBatchSize);
        innerBag.suAdapter.getPipelines(query, function (err, pipelines) {
            if (err) {
                logger.warn(who, util.format('Failed to get pipelines for query: %s with err: %s', query, err));
                return done(err);
            }
            innerBag.pipelines.push(...pipelines);
            return done();
        });
    }, function (err) {
        return next(err);
    });
}
function __getDisabledPipelineEntities(innerBag, next) {
    const who = innerBag.who + '|' + __getDisabledPipelineEntities.name;
    logger.debug(who, 'Inside');
    let projectIds = new Set();
    innerBag.pipelines.forEach(function (pipeline) {
        projectIds.add(pipeline.projectId);
    });
    const projectIdsString = [...projectIds].toString();
    innerBag.suAdapter.getDisabledPipelineEntities(`projectIds=${projectIdsString}&type=pipeline`, function (err, disabledPipelineEntities) {
        if (err) {
            logger.warn(who, util.format('Failed to get disabled pipeline entities with error: %s', err));
            return next(err);
        }
        innerBag.disabledPipelineEntities = disabledPipelineEntities;
        return next();
    });
}
function __checkIfPipelineDisabled(innerBag, next) {
    if (_.isEmpty(innerBag.disabledPipelineEntities)) {
        return next();
    }
    const who = innerBag.who + '|' + __checkIfPipelineDisabled.name;
    logger.debug(who, 'Inside');
    const pipelines = innerBag.pipelines;
    const disabledPipelinesEntities = innerBag.disabledPipelineEntities;
    for (let pipelinesIndex = 0; pipelinesIndex < pipelines.length; pipelinesIndex++) {
        const { id, projectId, name, pipelineSourceBranch } = pipelines[pipelinesIndex];
        for (let disabledPipelinesIndex = 0; disabledPipelinesIndex < disabledPipelinesEntities.length; disabledPipelinesIndex++) {
            const disabledPipelineEntity = disabledPipelinesEntities[disabledPipelinesIndex];
            if (projectId !== disabledPipelineEntity.projectId) {
                continue;
            }
            let isBranchPatternMatch = true;
            if (pipelineSourceBranch && disabledPipelineEntity?.branchPattern) {
                const branchPatternRegex = new RegExp(disabledPipelineEntity.branchPattern);
                isBranchPatternMatch = branchPatternRegex.test(pipelineSourceBranch);
            }
            const namePatternRegex = new RegExp(disabledPipelineEntity.namePattern);
            const isNamePatternMatch = namePatternRegex.test(name);
            if (!isNamePatternMatch || !isBranchPatternMatch) {
                continue;
            }
            const message = `Pipeline with id:${id} is disabled via disabledPipelineId:${disabledPipelineEntity.id} which has a branchPattern:${disabledPipelineEntity.branchPattern} and a pipelinePattern:${disabledPipelineEntity.namePattern}`;
            logger.info(who, message);
            innerBag.disabledPipelineIds.push(id);
            break;
        }
    }
    return next();
}
function _createRuns(bag, next) {
    if (_.isEmpty(bag.triggerablePipelineSteps))
        return next();
    var who = bag.who + '|' + _createRuns.name;
    logger.debug(who, 'Inside');
    var pipelineStepIdsByPipelineId = _.groupBy(bag.triggerablePipelineSteps, 'pipelineId');
    const startRunTime = Date.now();
    async.eachOfSeries(pipelineStepIdsByPipelineId, function (stepList, pipelineId, nextTriggerablePipeline) {
        createRun(bag.jFrogPipelinesSessionId, bag.inPayload, stepList, pipelineId, bag.triggersByPipelineStepId, function (err, runData) {
            const stopRunTime = Date.now();
            const runTimeDiff = new Date(stopRunTime - startRunTime);
            const runMS = runTimeDiff.getMilliseconds();
            const triggeredBy = runData?.triggeredByIdentity?.name || runData?.triggeredByResourceName;
            const runEvent = {
                domain: 'run',
                eventType: 'runCreated',
                status: err ? 'success' : 'failed',
                branchName: runData?.pipelineSource?.branch || 'Non-SCM',
                pipelineName: runData?.pipelineSource?.name || 'undefined',
                runNumber: runData?.run?.runNumber || 0,
                triggeredBy: triggeredBy || 'undefined',
                durationMillis: runTimeDiff.valueOf(),
                projectKey: runData?.project?.name || 'undefined',
                stepCount: bag.inPayload.pipelineStepIds?.length || 0,
                connectionCount: runData?.connections?.length || 0,
                hasHookUsed: runData?.preRunHookStep || runData?.postRunHookStep ? true : false,
                jFrogPipelinesSessionId: bag.jFrogPipelinesSessionId
            };
            bag.runs.push(runEvent);
            return process.nextTick(nextTriggerablePipeline);
        });
    }, function () {
        return next();
    });
}
//# sourceMappingURL=microWorker.js.map