"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.DependencyGraph = void 0;
const utility_1 = require("@jfrog-pipelines/utility");
const logger_1 = require("@jfrog-pipelines/logger");
const IN = 'IN';
const OUT = 'OUT';
class DependencyGraph {
    constructor(inputSteps) {
        this.inputSteps = inputSteps;
        this.logger = logger_1.LoggerFactory.getLogger('DependencyGraph');
    }
    populate() {
        this.logger.debug('Started');
        const dependencyNodes = this.initStepDependencyNodes();
        const connectedDependencyNodes = this.initConnectionDependencies(dependencyNodes);
        const connectedDependencies = this.populateResourceIO(connectedDependencyNodes);
        const stepData = connectedDependencies.filter((node) => {
            return node.step != null;
        });
        this.logger.debug('Completed');
        return (0, utility_1.keyBy)(stepData, 'step.id');
    }
    initStepDependencyNodes() {
        let dependencyAttributes = [];
        const steps = this.inputSteps.getSteps();
        dependencyAttributes = steps.map((step) => {
            return {
                step: step,
                inputs: [],
                outputs: [],
                runResourceVersion: null
            };
        });
        return dependencyAttributes;
    }
    initConnectionDependencies(stepDependencyAttributes) {
        const dependencyAttributes = [];
        const stepDependencyAttributesByStepId = (0, utility_1.keyBy)(stepDependencyAttributes, 'step.id');
        for (const stepDependencyAttribute of stepDependencyAttributes) {
            const stepId = stepDependencyAttribute.step?.id;
            const stepConnections = this.inputSteps.getRunStepConnectionsByStepId(stepId);
            if (!stepConnections) {
                dependencyAttributes.push(stepDependencyAttribute);
                continue;
            }
            this.processStepConnections(stepConnections, stepDependencyAttribute, stepDependencyAttributesByStepId, dependencyAttributes);
        }
        return dependencyAttributes.filter((step) => !(0, utility_1.isEmpty)(step));
    }
    processStepConnections(stepConnections, stepDependencyAttribute, stepDependencyAttributesByStepId, dependencyAttributes) {
        for (const stepConnection of stepConnections) {
            const stepGraphNode = stepDependencyAttribute;
            if (!this.hasValidOperationId(stepConnection.operationStepId, stepConnection.operationRunResourceVersionId)) {
                continue;
            }
            this.createConnectedStepNode(stepConnection, stepGraphNode, stepDependencyAttributesByStepId);
            const connectedResourceGraphNode = this.createConnectedResource(stepConnection, stepGraphNode);
            dependencyAttributes.push(connectedResourceGraphNode, stepGraphNode);
        }
    }
    createConnectedResource(stepConnection, stepGraphNode) {
        const connectedResource = this.inputSteps.getRunResourceVersionById(stepConnection.operationRunResourceVersionId);
        if (!connectedResource) {
            return {};
        }
        const connectedResourceGraphNode = {
            step: null,
            inputs: [],
            outputs: [],
            runResourceVersion: connectedResource
        };
        this.populateConnectionIO(stepConnection, stepGraphNode, connectedResourceGraphNode);
        return connectedResourceGraphNode;
    }
    createConnectedStepNode(stepConnection, stepGraphNode, stepMap) {
        const connectedStep = stepMap[stepConnection.operationStepId];
        if (!connectedStep) {
            return;
        }
        this.populateConnectionIO(stepConnection, stepGraphNode, connectedStep);
    }
    populateConnectionIO(stepConnection, parentGraphNode, childGraphNode) {
        switch (stepConnection.operation) {
            case IN: {
                parentGraphNode.inputs.push(childGraphNode);
                childGraphNode.outputs.push(parentGraphNode);
                break;
            }
            case OUT: {
                parentGraphNode.outputs.push(childGraphNode);
                childGraphNode.inputs.push(parentGraphNode);
                break;
            }
            default: {
                console.log('invalid op');
            }
        }
    }
    populateResourceIO(graphNodes) {
        const runResourceNodes = graphNodes.filter((node) => {
            return node.step === null;
        });
        const runResourceNodesByName = (0, utility_1.groupBy)(runResourceNodes, 'runResourceVersion.resourceName');
        const resourceNames = (0, utility_1.keys)(runResourceNodesByName);
        for (const resourceName of resourceNames) {
            this.processResourceInputOutputs(runResourceNodesByName[resourceName]);
        }
        return graphNodes;
    }
    processResourceInputOutputs(runResourceNodes) {
        const resourceInputs = runResourceNodes.flatMap((node) => node.inputs);
        const resourceOutputs = runResourceNodes.flatMap((node) => node.outputs);
        for (const resourceInput of resourceInputs) {
            resourceInput.outputs.push(...resourceOutputs);
        }
        for (const resourceOutput of resourceOutputs) {
            resourceOutput.inputs.push(...resourceInputs);
        }
    }
    hasValidOperationId(operationStepId, operationRunResourceVersionId) {
        return operationStepId > 0 || operationRunResourceVersionId > 0;
    }
}
exports.DependencyGraph = DependencyGraph;
//# sourceMappingURL=DependencyGraph.js.map