"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ChronologicalAssembler = void 0;
const metadata_1 = require("@jfrog-pipelines/metadata");
const utility_1 = require("@jfrog-pipelines/utility");
const logger_1 = require("@jfrog-pipelines/logger");
const Chronological_1 = require("./Chronological");
class ChronologicalAssembler {
    constructor(stepAttributes, pipeline, run, dataLoader) {
        this.stepAttributes = stepAttributes;
        this.pipeline = pipeline;
        this.run = run;
        this.dataLoader = dataLoader;
        this.logger = logger_1.LoggerFactory.getLogger('ChronologicalAssembler');
    }
    async populate() {
        this.logger.debug('Started');
        const chronologicalSteps = this.stepAttributes.filter((step) => {
            if (!(0, utility_1.isEmpty)(step.configPropertyBag)) {
                return step.configPropertyBag.chronological === true;
            }
            return false;
        });
        this.logger.trace('Chronological steps', chronologicalSteps.length);
        const chronologicalStepsByPipelineStepId = (0, utility_1.keyBy)(chronologicalSteps, 'pipelineStepId');
        const chronological = new Chronological_1.Chronological(chronologicalSteps, chronologicalStepsByPipelineStepId);
        this.logger.trace('Chronological pipeline flag enabled?', this.isChronologicalPipelineEnabled());
        chronological.setPipelineFlagEnabled(this.isChronologicalPipelineEnabled());
        chronological.setPreviousSteps(await this.getPreviousSteps(chronologicalStepsByPipelineStepId));
        if (this.isChronologicalPipelineEnabled()) {
            chronological.setPreviousRun(await this.getPreviousRun());
        }
        this.logger.debug('Completed');
        return chronological;
    }
    isChronologicalPipelineEnabled() {
        const pipelineConfig = this.pipeline.ymlConfigPropertyBag;
        if (!(0, utility_1.isEmpty)(pipelineConfig)) {
            return pipelineConfig.chronological === true;
        }
        return false;
    }
    async getPreviousSteps(chronologicalStepsByPipelineStepId) {
        const pipelineStepIds = (0, utility_1.keys)(chronologicalStepsByPipelineStepId);
        if (pipelineStepIds.length === 0) {
            return [];
        }
        const incompleteStatusCodes = metadata_1.NON_TERMINAL_RUN_STATUS_BY_CODE.join(',');
        const includeParameters = 'id,statusCode,pipelineStepId,runId';
        const query = `pipelineStepIds=${pipelineStepIds.join(',')}&statusCodes=${incompleteStatusCodes}&include=${includeParameters}`;
        const result = await this.dataLoader.getChronologicalSteps(query);
        return this.mapChronologicalSteps(result, chronologicalStepsByPipelineStepId);
    }
    mapChronologicalSteps(previousSteps, chronologicalStepsByPipelineStepId) {
        let chronologicalSteps = new Map();
        for (const previousRunStep of previousSteps) {
            if (previousRunStep.runId === this.run.id) {
                continue;
            }
            const currentRunStep = chronologicalStepsByPipelineStepId[previousRunStep.pipelineStepId];
            if (currentRunStep === undefined) {
                continue;
            }
            chronologicalSteps = this.filterPreviousStep(chronologicalSteps, previousRunStep, currentRunStep);
        }
        return [...chronologicalSteps.values()];
    }
    filterPreviousStep(chronologicalSteps, previousRunStep, currentRunStep) {
        if (currentRunStep.runId > previousRunStep.runId) {
            const mappedStep = chronologicalSteps.get(previousRunStep.pipelineStepId);
            if (mappedStep && mappedStep.previousStep.runId < previousRunStep.runId) {
                chronologicalSteps.set(previousRunStep.pipelineStepId, {
                    step: currentRunStep,
                    previousStep: previousRunStep
                });
                return chronologicalSteps;
            }
            chronologicalSteps.set(previousRunStep.pipelineStepId, {
                step: currentRunStep,
                previousStep: previousRunStep
            });
        }
        return chronologicalSteps;
    }
    async getPreviousRun() {
        const pipelineId = this.run.pipelineId;
        const incompleteStatusCodes = [metadata_1.CREATING, metadata_1.READY, metadata_1.WAITING, metadata_1.PROCESSING, metadata_1.CANCELLING, metadata_1.TIMINGOUT].join(',');
        const includeParameters = 'id,runNumber';
        const query = `pipelineIds=${pipelineId}&statusCodes=${incompleteStatusCodes}&include=${includeParameters}`;
        const previousRuns = await this.dataLoader.getRuns(query);
        let previousRun = null;
        for (const run of previousRuns) {
            if (run.id === this.run.id) {
                continue;
            }
            previousRun = this.filterPreviousRun(previousRun, run);
        }
        return previousRun;
    }
    filterPreviousRun(previousRun, currentRun) {
        if (!previousRun || currentRun.runNumber > previousRun.runNumber) {
            return currentRun;
        }
        return previousRun;
    }
}
exports.ChronologicalAssembler = ChronologicalAssembler;
//# sourceMappingURL=ChronologicalAssembler.js.map