"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.DependencyModeAssembler = void 0;
const utility_1 = require("@jfrog-pipelines/utility");
const logger_1 = require("@jfrog-pipelines/logger");
const DependencyMode_1 = require("./DependencyMode");
const { CALL_OBJECT_LIMIT_FOR_BATCH } = require('../../constants.js');
class DependencyModeAssembler {
    constructor(pipeline, dataLoader) {
        this.pipeline = pipeline;
        this.dataLoader = dataLoader;
        this.logger = logger_1.LoggerFactory.getLogger('DependencyModeAssembler');
        this.dependencyModePipelines = {
            waitOnChildCompletePipelineIds: [],
            waitOnParentCompletePipelineIds: [],
            waitOnParentSuccessPipelineIds: []
        };
    }
    async populate() {
        this.logger.debug('Started');
        const dependencyMode = new DependencyMode_1.DependencyMode();
        dependencyMode.setDependencyModeSettings(this.dependencyMode);
        this.logger.trace('dependencyModeSettings', this.dependencyMode);
        dependencyMode.setIsDependencyModeEnabled(this.isDependencyModeEnabled());
        this.logger.trace('Is dependencyMode enabled?', this.isDependencyModeEnabled());
        dependencyMode.setDependencyModePipelines(await this.getDependencyModePipelines());
        this.logger.debug('Completed');
        return dependencyMode;
    }
    isDependencyModeEnabled() {
        const dependencyMode = this.dependencyMode;
        return [
            dependencyMode.waitOnChildComplete,
            dependencyMode.waitOnParentComplete,
            dependencyMode.waitOnParentSuccess
        ].includes(true);
    }
    get dependencyMode() {
        const configPropertyBag = this.pipeline.ymlConfigPropertyBag;
        if ((0, utility_1.isEmpty)(configPropertyBag)) {
            return {};
        }
        const dependencyMode = configPropertyBag.dependencyMode;
        if ((0, utility_1.isEmpty)(dependencyMode)) {
            return {};
        }
        return dependencyMode;
    }
    async getDependencyModePipelines() {
        const dependencyModePipelines = {
            waitOnChildCompletePipelineIds: [],
            waitOnParentCompletePipelineIds: [],
            waitOnParentSuccessPipelineIds: []
        };
        if (!this.isDependencyModeEnabled()) {
            return dependencyModePipelines;
        }
        const dependencyMode = this.dependencyMode;
        const pipelineStepConnections = await this.getPipelineStepConnectionsForPipelineId();
        const potentialInputs = this.getPotentialInputs(pipelineStepConnections);
        const potentialOutputs = this.getPotentialOutputs(pipelineStepConnections);
        const potentialConnectingResourceIds = this.computePotentialConnectingResourceIds(dependencyMode, potentialInputs, potentialOutputs);
        const resourceIdChunks = (0, utility_1.chunk)(potentialConnectingResourceIds, CALL_OBJECT_LIMIT_FOR_BATCH);
        const allInputConnections = [...potentialInputs];
        const allOutputConnections = [...potentialOutputs];
        const pipelineStepConnectionsForResourceIds = await Promise.all(resourceIdChunks.map((resourceIds) => {
            return this.getInputAndOutputPipelineStepConnections(resourceIds);
        }));
        for (const { inputConnections, outputConnections } of pipelineStepConnectionsForResourceIds) {
            allInputConnections.push(...inputConnections);
            allOutputConnections.push(...outputConnections);
        }
        const inputConnectionsByPipelineId = (0, utility_1.groupBy)(allInputConnections, 'pipelineId');
        const inputConnectionsByResourceId = (0, utility_1.groupBy)(allInputConnections, 'operationResourceId');
        const outputConnectionsByPipelineId = (0, utility_1.groupBy)(allOutputConnections, 'pipelineId');
        const outputConnectionsByResourceId = (0, utility_1.groupBy)(allOutputConnections, 'operationResourceId');
        const parentPipelineIds = this.computeParentPipelines(inputConnectionsByPipelineId, outputConnectionsByResourceId);
        const childPipelineIds = this.computeChildPipelines(outputConnectionsByPipelineId, inputConnectionsByResourceId);
        const connectedPipelines = (0, utility_1.uniq)([...parentPipelineIds, ...childPipelineIds]);
        if (connectedPipelines.length === 0) {
            return dependencyModePipelines;
        }
        this.computeDependencyModePipelines(dependencyMode, parentPipelineIds, childPipelineIds, dependencyModePipelines);
        return dependencyModePipelines;
    }
    computePotentialConnectingResourceIds(dependencyMode, potentialInputs, potentialOutputs) {
        const potentialConnectingResourceIds = [];
        if (dependencyMode?.waitOnParentComplete || dependencyMode?.waitOnParentSuccess) {
            potentialConnectingResourceIds.push(...(0, utility_1.map)(potentialInputs, 'operationResourceId'));
        }
        if (dependencyMode?.waitOnChildComplete) {
            potentialConnectingResourceIds.push(...(0, utility_1.map)(potentialOutputs, 'operationResourceId'));
        }
        return potentialConnectingResourceIds;
    }
    async getInputAndOutputPipelineStepConnections(resourceIds) {
        const query = `operationResourceIds=${resourceIds.join(',')}&sortOrder=1&sortBy=id`;
        const pipelineStepConnections = await this.dataLoader.getPipelineStepConnections(query);
        const potentialInputs = this.getPotentialInputs(pipelineStepConnections);
        const potentialInputsByResourceId = (0, utility_1.groupBy)(potentialInputs, 'operationResourceId');
        const potentialOutputs = this.getPotentialOutputs(pipelineStepConnections);
        const potentialOutputsByResourceId = (0, utility_1.groupBy)(potentialOutputs, 'operationResourceId');
        for (const resourceId of resourceIds) {
            this.computePotentialInputsAndOutputsForResourceId(resourceId, potentialInputsByResourceId, potentialOutputsByResourceId);
        }
        return {
            inputConnections: (0, utility_1.flatten)(Object.values(potentialInputsByResourceId)),
            outputConnections: (0, utility_1.flatten)(Object.values(potentialOutputsByResourceId))
        };
    }
    computePotentialInputsAndOutputsForResourceId(resourceId, potentialInputsByResourceId, potentialOutputsByResourceId) {
        const inputPipelineIds = (0, utility_1.uniq)((0, utility_1.map)(potentialInputsByResourceId[resourceId], 'pipelineId'));
        const outputPipelineIds = (0, utility_1.uniq)((0, utility_1.map)(potentialOutputsByResourceId[resourceId], 'pipelineId'));
        if (this.checkCommonPipelines(inputPipelineIds, outputPipelineIds)) {
            potentialInputsByResourceId[resourceId] = [];
            potentialOutputsByResourceId[resourceId] = [];
        }
    }
    checkCommonPipelines(inputPipelineIds, outputPipelineIds) {
        if (inputPipelineIds.length === 0 && outputPipelineIds.length === 0) {
            return true;
        }
        if (inputPipelineIds.length !== outputPipelineIds.length) {
            return false;
        }
        const commonPipelineIds = (0, utility_1.intersection)(inputPipelineIds, outputPipelineIds);
        return commonPipelineIds.length === inputPipelineIds.length;
    }
    computeParentPipelines(inputConnectionsByPipelineId, outputConnectionsByResourceId) {
        const inputResourceIds = (0, utility_1.map)(inputConnectionsByPipelineId[this.pipeline.id], 'operationResourceId');
        let parentOutputPipelineIds = [];
        for (const inputResourceId of inputResourceIds) {
            parentOutputPipelineIds = (0, utility_1.union)(parentOutputPipelineIds, (0, utility_1.map)(outputConnectionsByResourceId[inputResourceId], 'pipelineId'));
        }
        return (0, utility_1.without)(parentOutputPipelineIds, this.pipeline.id);
    }
    computeChildPipelines(outputConnectionsByPipelineId, inputConnectionsByResourceId) {
        const outputResourceIds = (0, utility_1.map)(outputConnectionsByPipelineId[this.pipeline.id], 'operationResourceId');
        let childInputPipelineIds = [];
        for (const outputResourceId of outputResourceIds) {
            childInputPipelineIds = (0, utility_1.union)(childInputPipelineIds, (0, utility_1.map)(inputConnectionsByResourceId[outputResourceId], 'pipelineId'));
        }
        return (0, utility_1.without)(childInputPipelineIds, this.pipeline.id);
    }
    computeWaitOnParentComplete(parentPipelineIds, dependencyModePipelines) {
        for (const parentPipelineId of parentPipelineIds) {
            dependencyModePipelines.waitOnParentCompletePipelineIds.push(parentPipelineId);
        }
    }
    computeWaitOnParentSuccess(parentPipelineIds, dependencyModePipelines) {
        for (const parentPipelineId of parentPipelineIds) {
            dependencyModePipelines.waitOnParentSuccessPipelineIds.push(parentPipelineId);
        }
    }
    computeWaitOnChildComplete(childPipelineIds, dependencyModePipelines) {
        for (const childPipelineId of childPipelineIds) {
            dependencyModePipelines.waitOnChildCompletePipelineIds.push(childPipelineId);
        }
    }
    computeDependencyModePipelines(dependencyMode, parentPipelineIds, childPipelineIds, dependencyModePipelines) {
        if (dependencyMode.waitOnParentComplete) {
            this.computeWaitOnParentComplete(parentPipelineIds, dependencyModePipelines);
        }
        if (dependencyMode.waitOnChildComplete) {
            this.computeWaitOnChildComplete(childPipelineIds, dependencyModePipelines);
        }
        if (dependencyMode.waitOnParentSuccess) {
            this.computeWaitOnParentSuccess(parentPipelineIds, dependencyModePipelines);
        }
    }
    getPipelineStepConnectionsForPipelineId() {
        const query = `pipelineIds=${this.pipeline.id}&sortOrder=1&sortBy=id`;
        return this.dataLoader.getPipelineStepConnections(query);
    }
    getPotentialInputs(pipelineStepConnections) {
        return pipelineStepConnections.filter((connection) => {
            return connection.operation === 'IN' && connection.isTrigger && connection.operationResourceId;
        });
    }
    getPotentialOutputs(pipelineStepConnections) {
        return pipelineStepConnections.filter((connection) => {
            return connection.operation === 'OUT' && connection.operationResourceId;
        });
    }
}
exports.DependencyModeAssembler = DependencyModeAssembler;
//# sourceMappingURL=DependencyModeAssembler.js.map